import time
import unittest
import threading
import vapoursynth as vs

from concurrent.futures import Future


class FilterTestSequence(unittest.TestCase):

    def setUp(self):
        self.core = vs.core
        self.filter = self.core.std.BlankClip(length=20)
        self.slow_filter = self.filter.std.FrameEval(self.slow_filter_fe)
        self.fail_filter = self.filter.std.FrameEval(self.fail_filter_fe)

        self.condition = threading.Event()

        self.cb_called = False
        self.cb_result = None
        self.cb_node = None
        self.cb_n = None

        self.mv_called = False

    def slow_filter_fe(self, n):
        time.sleep(1)
        return self.filter

    def fail_filter_fe(self, n):
        time.sleep(1)
        raise RuntimeError("Fail")

    def cb_old(self, node, n, result):
        self.cb_called = True
        self.cb_result = result
        self.cb_node = node
        self.cb_n = n

        self.condition.set()

    def cb(self, result, error):
        self.cb_result = (result, error)
        self.cb_called = True
        self.cb_node = None
        self.cb_n = None

        self.condition.set()

    def mv(self, func, *args, **kwargs):
        self.mv_called = func
        return func(*args, **kwargs)

    ##########################################################################
    # Tests start here

    def test_raw_cb_slow(self):
        self.slow_filter.get_frame_async(0, self.cb)
        self.condition.wait(2)

        self.assertTrue(self.cb_called)
        self.assertIsInstance(self.cb_result, tuple)
        self.assertEqual(len(self.cb_result), 2)
        self.assertIsInstance(self.cb_result[0], vs.VideoFrame)
        self.assertIsNone(self.cb_result[1])

    def test_raw_cb_fail(self):
        self.fail_filter.get_frame_async(0, self.cb)
        self.condition.wait(2)

        self.assertTrue(self.cb_called)
        self.assertIsInstance(self.cb_result, tuple)
        self.assertEqual(len(self.cb_result), 2)
        self.assertIsInstance(self.cb_result[1], vs.Error)
        self.assertIsNone(self.cb_result[0])

    def test_async_slow(self):
        fut = self.slow_filter.get_frame_async(1)
        self.assertIsInstance(fut.result(2), vs.VideoFrame)

    def test_async_fail(self):
        fut = self.fail_filter.get_frame_async(1)
        with self.assertRaisesRegex(vs.Error, "Fail"):
            fut.result(2)

if __name__ == '__main__':
    unittest.main()



import unittest
import vapoursynth as vs

def float_lut(x):
    return float(1)

def int_lut(x):
    return int(1)

class CoreTestSequence(unittest.TestCase):

    def setUp(self):
        self.core = vs.core
        self.core.num_threads = 10

    # core argument tests
    def test_num_threads(self):
        self.assertEqual(self.core.num_threads, 10)

    def test_create_video_frame(self):
        frame = self.core.create_video_frame(vs.RGB24, 100, 100)
        self.assertIsInstance(frame, vs.VideoFrame)
        self.assertEqual(frame.format, self.core.get_video_format(vs.RGB24))
        self.assertEqual(frame.width, 100)
        self.assertEqual(frame.height, 100)
        self.assertFalse(frame.readonly)

### Clip-Attr tests

    def test_frames_generator(self):
        clip = self.core.std.BlankClip(length=200)
        e = -1
        for e, frame in enumerate(clip.frames()):
            self.assertIsInstance(frame, vs.VideoFrame)
        self.assertEqual(e, 199)

### Filter-Call-Tests

    def test_func1(self):
        with self.assertRaises(AttributeError):
            self.core.blah.list_functions()

    def test_arg1(self):
        self.core.std.BlankClip()

    def test_arg2(self):
        self.core.std.BlankClip(width=50)

    def test_arg3(self):
        self.core.std.BlankClip(width=[50])

    def test_arg4(self):
        with self.assertRaises(vs.Error):
            self.core.std.BlankClip(width=[50, 50])

    def test_arg5(self):
        with self.assertRaises(vs.Error):
            self.core.std.BlankClip(width=[])

    def test_arg6(self):
        self.core.std.BlankClip(width_=50)

    def test_arg7(self):
        with self.assertRaises(vs.Error):
            self.core.std.BlankClip(width_=[])

    def test_arg8(self):
        with self.assertRaises(vs.Error):
            self.core.std.BlankClip(10,10,10,10,10,10,10,10,10,10,10,10,10,10)

    def test_arg9(self):
        with self.assertRaises(vs.Error):
            self.core.std.BlankClip(width2=50)

    def test_arg10(self):
        with self.assertRaises(vs.Error):
            self.core.std.FlipVertical()

    def test_arg11(self):
        clip = self.core.std.BlankClip(format=vs.YUV420P8, color=[69, 242, 115])
        self.core.text.ClipInfo(clip)

    def test_arg12(self):
        clip = self.core.std.BlankClip(format=vs.YUV420P8, color=[69, 242, 115])
        self.core.text.ClipInfo(clip, 5)

    def test_arg13(self):
        clip = self.core.std.BlankClip(format=vs.YUV420P8, color=[69, 242, 115])
        with self.assertRaises(vs.Error):
            self.core.text.ClipInfo(clip, alignment2=None)

    def test_arg14(self):
        clip = self.core.std.BlankClip(format=vs.YUV420P8, color=[69, 242, 115])
        self.core.text.ClipInfo(clip, alignment=None)

#blank clip argument tests
    def test_blank_clip_no_variable(self):
        clip = self.core.std.BlankClip(format=vs.YUV444P8, color=[69, 242, 115], width=120, height=121)
        self.assertEqual(clip.width, 120)
        self.assertEqual(clip.height, 121)
        self.assertEqual(int(clip.format), int(vs.YUV444P8))

        frame = clip.get_frame(0)
        self.assertEqual(frame.width, 120)
        self.assertEqual(frame.height, 121)
        self.assertEqual(int(frame.format), int(vs.YUV444P8))

    def test_blank_clip_variable_size(self):
        clip = self.core.std.BlankClip(format=vs.YUV444P8, color=[69, 242, 115], varsize=True, width=120, height=121)
        self.assertEqual(clip.width, 0)
        self.assertEqual(clip.height, 0)
        self.assertEqual(int(clip.format), int(vs.YUV444P8))

        frame = clip.get_frame(0)
        self.assertEqual(frame.width, 120)
        self.assertEqual(frame.height, 121)
        self.assertEqual(int(frame.format), int(vs.YUV444P8))

    def test_blank_clip_variable_format(self):
        clip = self.core.std.BlankClip(format=vs.YUV444P8, color=[69, 242, 115], varformat=True, width=120, height=121)
        self.assertEqual(clip.width, 120)
        self.assertEqual(clip.height, 121)
        self.assertEqual(clip.format, None)

        frame = clip.get_frame(0)
        self.assertEqual(frame.width, 120)
        self.assertEqual(frame.height, 121)
        self.assertEqual(int(frame.format), int(vs.YUV444P8))


#lut argument tests
    def test_lut_arg1(self):
        clip = self.core.std.BlankClip(format=vs.YUV420P8, color=[69, 242, 115])
        with self.assertRaises(vs.Error):
            self.core.std.Lut(clip, lut=list(range(256)), function=lambda x: x)

    def test_lut_arg2(self):
        clip = self.core.std.BlankClip(format=vs.YUV420P8, color=[69, 242, 115])
        with self.assertRaises(vs.Error):
            self.core.std.Lut(clip)

    def test_lut_arg3(self):
        clip = self.core.std.BlankClip(format=vs.YUV420P8, color=[69, 242, 115])
        self.core.std.Lut(clip, lut=list(range(256)))

    def test_lut_arg4(self):
        clip = self.core.std.BlankClip(format=vs.YUV420P8, color=[69, 242, 115])
        with self.assertRaises(vs.Error):
            self.core.std.Lut(clip, lut=list(range(257)))

    def test_lut_arg5(self):
        clip = self.core.std.BlankClip(format=vs.YUV420P8, color=[69, 242, 115])
        with self.assertRaises(vs.Error):
            self.core.std.Lut(clip, lut=list(range(255)))

    def test_lut_arg6(self):
        clip = self.core.std.BlankClip(format=vs.YUV420P8, color=[69, 242, 115])
        with self.assertRaises(vs.Error):
            self.core.std.Lut(clip, lut=list(range(255)), function=lambda x: x)

    def test_lut_arg7(self):
        clip = self.core.std.BlankClip(format=vs.YUV420P8, color=[69, 242, 115])
        with self.assertRaises(vs.Error):
            self.core.std.Lut(clip, lut=list(range(255)), function=lambda x: x + 1)

    def test_lut_arg8(self):
        clip = self.core.std.BlankClip(format=vs.YUV420P8, color=[69, 242, 115])
        self.core.std.Lut(clip, function=lambda x: x)

    def test_lut_arg9(self):
        clip = self.core.std.BlankClip(format=vs.YUV420P8, color=[69, 242, 115])
        with self.assertRaises(vs.Error):
            self.core.std.Lut(clip, function=lambda x: x, planes=[3])
        
    def test_lut_arg10(self):
        clip = self.core.std.BlankClip(format=vs.YUV420P8, color=[69, 242, 115])
        with self.assertRaises(vs.Error):
            self.core.std.Lut(clip, function=float_lut, floatout=False)
            
    def test_lut_arg11(self):
        clip = self.core.std.BlankClip(format=vs.YUV420P8, color=[69, 242, 115])
        self.core.std.Lut(clip, function=float_lut, floatout=True)
        
    def test_lut_arg12(self):
        clip = self.core.std.BlankClip(format=vs.YUV420P8, color=[69, 242, 115])
        with self.assertRaises(vs.Error):
            self.core.std.Lut(clip, function=int_lut, floatout=True)
            
    def test_lut_arg13(self):
        clip = self.core.std.BlankClip(format=vs.YUV420P8, color=[69, 242, 115])
        self.core.std.Lut(clip, function=int_lut, floatout=False)

#lut2 argument tests
    def test_lut2_arg1(self):
        clip = self.core.std.BlankClip(format=vs.YUV420P8, color=[69, 242, 115])
        with self.assertRaises(vs.Error):
            self.core.std.Lut2(clip, clip, lut=[5]*65536, function=lambda x,y: (x+y)//2)

    def test_lut2_arg2(self):
        clip = self.core.std.BlankClip(format=vs.YUV420P8, color=[69, 242, 115])
        with self.assertRaises(vs.Error):
            self.core.std.Lut2(clip, clip)

    def test_lut2_arg3(self):
        clip = self.core.std.BlankClip(format=vs.YUV420P8, color=[69, 242, 115])
        self.core.std.Lut2(clip, clip, lut=[5]*65536)

    def test_lut2_arg4(self):
        clip = self.core.std.BlankClip(format=vs.YUV420P8, color=[69, 242, 115])
        with self.assertRaises(vs.Error):
            self.core.std.Lut2(clip, clip, lut=[5]*65537)

    def test_lut2_arg5(self):
        clip = self.core.std.BlankClip(format=vs.YUV420P8, color=[69, 242, 115])
        with self.assertRaises(vs.Error):
            self.core.std.Lut2(clip, clip, lut=[5]*65535)

    def test_lut2_arg6(self):
        clip = self.core.std.BlankClip(format=vs.YUV420P8, color=[69, 242, 115])
        with self.assertRaises(vs.Error):
            self.core.std.Lut2(clip, clip, lut=[5]*65535, function=lambda x,y: (x+y)//2)

    def test_lut2_arg7(self):
        clip = self.core.std.BlankClip(format=vs.YUV420P8, color=[69, 242, 115])
        with self.assertRaises(vs.Error):
            self.core.std.Lut2(clip, clip, lut=[5]*65535, function=lambda x,y: x*y)

    def test_lut2_arg8(self):
        clip = self.core.std.BlankClip(format=vs.YUV420P8, color=[69, 242, 115])
        self.core.std.Lut2(clip, clip, function=lambda x,y: (x+y)//2)

    def test_lut2_arg9(self):
        clip = self.core.std.BlankClip(format=vs.YUV420P8, color=[69, 242, 115])
        with self.assertRaises(vs.Error):
            self.core.std.Lut2(clip, clip, function=lambda x,y: (x+y)//2, planes=[3])
            
#shuffleplanes argument tests
    def test_suffleplanes_arg1(self):
        clip = self.core.std.BlankClip(format=vs.YUV420P8)
        with self.assertRaises(vs.Error):
            self.core.std.ShufflePlanes(clip, planes=[0, 1, 2], colorfamily=vs.RGB)

    def test_suffleplanes_arg2(self):
        clip = self.core.std.BlankClip(format=vs.YUV420P8)
        self.core.std.ShufflePlanes(clip, planes=[0, 1, 2], colorfamily=vs.YUV)

    def test_suffleplanes_arg3(self):
        clip = self.core.std.BlankClip(format=vs.YUV420P8)
        self.core.std.ShufflePlanes(clip, planes=[1, 1, 2], colorfamily=vs.RGB)

    def test_suffleplanes_arg4(self):
        clip1 = self.core.std.BlankClip(format=vs.YUV420P8)
        clip2 = self.core.std.BlankClip(format=vs.YUV420P10)
        with self.assertRaises(vs.Error):
            self.core.std.ShufflePlanes([clip1, clip2], planes=[0, 1, 2], colorfamily=vs.YUV)

    def test_suffleplanes_arg5(self):
        clip1 = self.core.std.BlankClip(format=vs.YUV420P8)
        clip2 = self.core.std.BlankClip(format=vs.RGB24)
        with self.assertRaises(vs.Error):
            self.core.std.ShufflePlanes([clip1, clip2], planes=[2, 1, 2], colorfamily=vs.RGB)

    def test_suffleplanes_arg6(self):
        clip1 = self.core.std.BlankClip(format=vs.YUV420P8)
        clip2 = self.core.std.BlankClip(format=vs.RGB24)
        with self.assertRaises(vs.Error):
            self.core.std.ShufflePlanes([clip1, clip2, clip1], planes=[0, 1, 2], colorfamily=vs.RGB)               

#clamp tests
    def test_levels_clamp(self):
        for i in range(1024):
            levels_clip = self.core.std.BlankClip(format=vs.YUV420P10, color=[i, 0, 0]).std.Levels(min_in=504, max_in=520, gamma=1, min_out=0, max_out=1023, planes=0).std.PEMVerifier()
            levels_clip.get_frame(0)

    def test_convolution_clamp(self):
        for i in range(1024):
            levels_clip = self.core.std.BlankClip(format=vs.YUV420P10, color=[i, 0, 0]).std.Convolution([1, 1, 1, 1, 1, 1, 1, 1, 1], divisor=.1, planes=0).std.PEMVerifier()
            levels_clip.get_frame(0)

if __name__ == '__main__':
    unittest.main()



#
# Copyright (c) 2020 Fredrik Mellbin
#
# This file is part of VapourSynth.
#
# VapourSynth is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# VapourSynth is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with VapourSynth; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#

from vapoursynth cimport VSCore

cdef extern from "src/vsscript/vsscript_internal.h" nogil:

    ctypedef struct VSScript:
        void *pyenvdict
        void *errstr
        VSCore *core
        int id
        int exitCode
        int setCWD



#
# Copyright (c) 2020 Fredrik Mellbin
#
# This file is part of VapourSynth.
#
# VapourSynth is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# VapourSynth is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with VapourSynth; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#

cdef extern from "include/VSScript4.h" nogil:
    enum:
        VSSCRIPT_API_MAJOR
        VSSCRIPT_API_MINOR
        VSSCRIPT_API_VERSION



import math
import unittest
import vapoursynth as vs

def get_pixel_value(clip):
    frame = clip.get_frame(0)
    arr = frame[0]
    return arr[0,0]

class CoreTestSequence(unittest.TestCase):

    def setUp(self):
        self.core = vs.core
        self.core.num_threads = 1
            
    def test_expr_op1(self):
        clip = self.core.std.BlankClip(format=vs.GRAY8, color=58)
        clip = self.core.std.Expr(clip, "x 2 *")
        self.assertEqual(get_pixel_value(clip), 116)

    def test_expr_op2(self):
        clip = self.core.std.BlankClip(format=vs.GRAY8, color=57)
        clip = self.core.std.Expr(clip, "x 2 /")
        self.assertEqual(get_pixel_value(clip), 28)

    def test_expr_op3(self):
        clip = self.core.std.BlankClip(format=vs.GRAY8, color=58)
        clip = self.core.std.Expr(clip, "x 2 / 0.1 +")
        self.assertEqual(get_pixel_value(clip), 29)

    def test_expr_op4(self):
        clip = self.core.std.BlankClip(format=vs.GRAY8, color=58)
        clip = self.core.std.Expr(clip, "x 10 +")
        self.assertEqual(get_pixel_value(clip), 68)

    def test_expr_op5(self):
        clip = self.core.std.BlankClip(format=vs.GRAY8, color=58)
        clip = self.core.std.Expr(clip, "x 28 -")
        self.assertEqual(get_pixel_value(clip), 30)

    def test_expr_op6(self):
        clip = self.core.std.BlankClip(format=vs.GRAY8, color=58)
        clip = self.core.std.Expr(clip, "x -1 * abs")
        self.assertEqual(get_pixel_value(clip), 58)

    def test_expr_op7(self):
        clip = self.core.std.BlankClip(format=vs.GRAY8, color=58)
        clip = self.core.std.Expr(clip, "x sqrt")
        self.assertEqual(get_pixel_value(clip), 8)

    def test_expr_op8(self):
        clip = self.core.std.BlankClip(format=vs.GRAY8, color=58)
        clip = self.core.std.Expr(clip, "x dup -")
        self.assertEqual(get_pixel_value(clip), 0)

    def test_expr_op9(self):
        clip = self.core.std.BlankClip(format=vs.GRAY8, color=58)
        clip = self.core.std.Expr(clip, "x dup +")
        self.assertEqual(get_pixel_value(clip), 116)

    def test_expr_op10(self):
        clip = self.core.std.BlankClip(format=vs.GRAY8, color=58)
        clip = self.core.std.Expr(clip, "2 x swap /")
        self.assertEqual(get_pixel_value(clip), 29)

    def test_expr_op11(self):
        clip = self.core.std.BlankClip(format=vs.GRAY8, color=58)
        clip = self.core.std.Expr(clip, "x 60 max")
        self.assertEqual(get_pixel_value(clip), 60)

    def test_expr_op12(self):
        clip = self.core.std.BlankClip(format=vs.GRAY8, color=58)
        clip = self.core.std.Expr(clip, "40 x min")
        self.assertEqual(get_pixel_value(clip), 40)

    def test_expr_op13(self):
        clip = self.core.std.BlankClip(format=vs.GRAY8, color=2)
        clip = self.core.std.Expr(clip, "x exp")
        self.assertEqual(get_pixel_value(clip), 7)

    def test_expr_op14(self):
        clip = self.core.std.BlankClip(format=vs.GRAY8, color=3)
        clip = self.core.std.Expr(clip, "x exp")
        self.assertEqual(get_pixel_value(clip), 20)

    def test_expr_op15(self):
        clip = self.core.std.BlankClip(format=vs.GRAY8, color=0)
        clip = self.core.std.Expr(clip, "x exp")
        self.assertEqual(get_pixel_value(clip), 1)

    def test_expr_op16(self):
        clip = self.core.std.BlankClip(format=vs.GRAY8, color=58)
        clip = self.core.std.Expr(clip, "x log")
        self.assertEqual(get_pixel_value(clip), 4)

    def test_expr_op17(self):
        clip = self.core.std.BlankClip(format=vs.GRAY8, color=58)
        clip = self.core.std.Expr(clip, "x log exp")
        self.assertEqual(get_pixel_value(clip), 58)

    def test_expr_op18(self):
        clip = self.core.std.BlankClip(format=vs.GRAY8, color=58)
        clip = self.core.std.Expr(clip, "x 10 <")
        self.assertEqual(get_pixel_value(clip), 0)

    def test_expr_op19(self):
        clip = self.core.std.BlankClip(format=vs.GRAY8, color=58)
        clip = self.core.std.Expr(clip, "10 x <")
        self.assertEqual(get_pixel_value(clip), 1)

    def test_expr_op20(self):
        clip = self.core.std.BlankClip(format=vs.GRAY8, color=58)
        clip = self.core.std.Expr(clip, "58 x <")
        self.assertEqual(get_pixel_value(clip), 0)

    def test_expr_op21(self):
        clip = self.core.std.BlankClip(format=vs.GRAY8, color=58)
        clip = self.core.std.Expr(clip, "x 58 <")
        self.assertEqual(get_pixel_value(clip), 0)

    def test_expr_op22(self):
        clip = self.core.std.BlankClip(format=vs.GRAY8, color=58)
        clip = self.core.std.Expr(clip, "10 x >")
        self.assertEqual(get_pixel_value(clip), 0)

    def test_expr_op23(self):
        clip = self.core.std.BlankClip(format=vs.GRAY8, color=58)
        clip = self.core.std.Expr(clip, "x 10 >")
        self.assertEqual(get_pixel_value(clip), 1)

    def test_expr_op24(self):
        clip = self.core.std.BlankClip(format=vs.GRAY8, color=58)
        clip = self.core.std.Expr(clip, "58 x >")
        self.assertEqual(get_pixel_value(clip), 0)

    def test_expr_op25(self):
        clip = self.core.std.BlankClip(format=vs.GRAY8, color=58)
        clip = self.core.std.Expr(clip, "x 58 >")
        self.assertEqual(get_pixel_value(clip), 0)

    def test_expr_op26(self):
        clip = self.core.std.BlankClip(format=vs.GRAY8, color=58)
        clip = self.core.std.Expr(clip, "x 10 <=")
        self.assertEqual(get_pixel_value(clip), 0)

    def test_expr_op27(self):
        clip = self.core.std.BlankClip(format=vs.GRAY8, color=58)
        clip = self.core.std.Expr(clip, "10 x <=")
        self.assertEqual(get_pixel_value(clip), 1)

    def test_expr_op28(self):
        clip = self.core.std.BlankClip(format=vs.GRAY8, color=58)
        clip = self.core.std.Expr(clip, "58 x <=")
        self.assertEqual(get_pixel_value(clip), 1)

    def test_expr_op29(self):
        clip = self.core.std.BlankClip(format=vs.GRAY8, color=58)
        clip = self.core.std.Expr(clip, "x 58 <=")
        self.assertEqual(get_pixel_value(clip), 1)

    def test_expr_op30(self):
        clip = self.core.std.BlankClip(format=vs.GRAY8, color=58)
        clip = self.core.std.Expr(clip, "10 x >=")
        self.assertEqual(get_pixel_value(clip), 0)

    def test_expr_op31(self):
        clip = self.core.std.BlankClip(format=vs.GRAY8, color=58)
        clip = self.core.std.Expr(clip, "x 10 >=")
        self.assertEqual(get_pixel_value(clip), 1)

    def test_expr_op32(self):
        clip = self.core.std.BlankClip(format=vs.GRAY8, color=58)
        clip = self.core.std.Expr(clip, "58 x >=")
        self.assertEqual(get_pixel_value(clip), 1)

    def test_expr_op33(self):
        clip = self.core.std.BlankClip(format=vs.GRAY8, color=58)
        clip = self.core.std.Expr(clip, "x 58 >=")
        self.assertEqual(get_pixel_value(clip), 1)

    def test_expr_op34(self):
        clip1 = self.core.std.BlankClip(format=vs.GRAY8, color=58)
        clip2 = self.core.std.BlankClip(format=vs.GRAY8, color=58)
        clip = self.core.std.Expr((clip1, clip2), "x y =")
        self.assertEqual(get_pixel_value(clip), 1)

    def test_expr_op35(self):
        clip = self.core.std.BlankClip(format=vs.GRAY8, color=4)
        clip = self.core.std.Expr(clip, "x x 1 - =")
        self.assertEqual(get_pixel_value(clip), 0)

    def test_expr_op36(self):
        clip1 = self.core.std.BlankClip(format=vs.GRAY8, color=0)
        clip2 = self.core.std.BlankClip(format=vs.GRAY8, color=58)
        clip = self.core.std.Expr((clip1, clip2), "x y and")
        self.assertEqual(get_pixel_value(clip), 0)

    def test_expr_op37(self):
        clip1 = self.core.std.BlankClip(format=vs.GRAY8, color=0)
        clip2 = self.core.std.BlankClip(format=vs.GRAY8, color=58)
        clip = self.core.std.Expr((clip1, clip2), "x y or")
        self.assertEqual(get_pixel_value(clip), 1)

    def test_expr_op38(self):
        clip1 = self.core.std.BlankClip(format=vs.GRAY8, color=0)
        clip2 = self.core.std.BlankClip(format=vs.GRAY8, color=58)
        clip = self.core.std.Expr((clip1, clip2), "x y xor")
        self.assertEqual(get_pixel_value(clip), 1)
        
    def test_expr_op39(self):
        clip1 = self.core.std.BlankClip(format=vs.GRAY8, color=1)
        clip2 = self.core.std.BlankClip(format=vs.GRAY8, color=0)
        clip = self.core.std.Expr((clip1, clip2), "x y and")
        self.assertEqual(get_pixel_value(clip), 0)

    def test_expr_op40(self):
        clip1 = self.core.std.BlankClip(format=vs.GRAY8, color=1)
        clip2 = self.core.std.BlankClip(format=vs.GRAY8, color=0)
        clip = self.core.std.Expr((clip1, clip2), "x y or")
        self.assertEqual(get_pixel_value(clip), 1)

    def test_expr_op41(self):
        clip1 = self.core.std.BlankClip(format=vs.GRAY8, color=1)
        clip2 = self.core.std.BlankClip(format=vs.GRAY8, color=0)
        clip = self.core.std.Expr((clip1, clip2), "x y xor")
        self.assertEqual(get_pixel_value(clip), 1)

    def test_expr_op42(self):
        clip1 = self.core.std.BlankClip(format=vs.GRAY8, color=0)
        clip2 = self.core.std.BlankClip(format=vs.GRAY8, color=0)
        clip = self.core.std.Expr((clip1, clip2), "x y and")
        self.assertEqual(get_pixel_value(clip), 0)

    def test_expr_op43(self):
        clip1 = self.core.std.BlankClip(format=vs.GRAY8, color=0)
        clip2 = self.core.std.BlankClip(format=vs.GRAY8, color=0)
        clip = self.core.std.Expr((clip1, clip2), "x y or")
        self.assertEqual(get_pixel_value(clip), 0)

    def test_expr_op44(self):
        clip1 = self.core.std.BlankClip(format=vs.GRAY8, color=0)
        clip2 = self.core.std.BlankClip(format=vs.GRAY8, color=0)
        clip = self.core.std.Expr((clip1, clip2), "x y xor")
        self.assertEqual(get_pixel_value(clip), 0)

    def test_expr_op45(self):
        clip1 = self.core.std.BlankClip(format=vs.GRAY8, color=8)
        clip2 = self.core.std.BlankClip(format=vs.GRAY8, color=7)
        clip = self.core.std.Expr((clip1, clip2), "x y and")
        self.assertEqual(get_pixel_value(clip), 1)

    def test_expr_op46(self):
        clip1 = self.core.std.BlankClip(format=vs.GRAY8, color=8)
        clip2 = self.core.std.BlankClip(format=vs.GRAY8, color=7)
        clip = self.core.std.Expr((clip1, clip2), "x y or")
        self.assertEqual(get_pixel_value(clip), 1)

    def test_expr_op47(self):
        clip1 = self.core.std.BlankClip(format=vs.GRAY8, color=8)
        clip2 = self.core.std.BlankClip(format=vs.GRAY8, color=7)
        clip = self.core.std.Expr((clip1, clip2), "x y xor")
        self.assertEqual(get_pixel_value(clip), 0)

    def test_expr_op48(self):
        clip1 = self.core.std.BlankClip(format=vs.GRAY8, color=100)
        clip2 = self.core.std.BlankClip(format=vs.GRAY8, color=200)
        clip3 = self.core.std.BlankClip(format=vs.GRAY8, color=0)
        clip = self.core.std.Expr((clip1, clip2, clip3), "z x y ?")
        self.assertEqual(get_pixel_value(clip), 200)

    def test_expr_op49(self):
        clip1 = self.core.std.BlankClip(format=vs.GRAY8, color=100)
        clip2 = self.core.std.BlankClip(format=vs.GRAY8, color=200)
        clip3 = self.core.std.BlankClip(format=vs.GRAY8, color=1)
        clip = self.core.std.Expr((clip1, clip2, clip3), "z x y ?")
        self.assertEqual(get_pixel_value(clip), 100)

    def test_expr_op50(self):
        clip1 = self.core.std.BlankClip(format=vs.GRAY8, color=100)
        clip2 = self.core.std.BlankClip(format=vs.GRAY8, color=200)
        clip3 = self.core.std.BlankClip(format=vs.GRAY8, color=0)
        clip = self.core.std.Expr((clip1, clip2, clip3), "z not x y ?")
        self.assertEqual(get_pixel_value(clip), 100)

    def test_expr_op51(self):
        clip1 = self.core.std.BlankClip(format=vs.GRAY8, color=100)
        clip2 = self.core.std.BlankClip(format=vs.GRAY8, color=200)
        clip3 = self.core.std.BlankClip(format=vs.GRAY8, color=100)
        clip = self.core.std.Expr((clip1, clip2, clip3), "z not x y ?")
        self.assertEqual(get_pixel_value(clip), 200)

    def test_expr_op52(self):
        clip = self.core.std.BlankClip(format=vs.GRAY8, color=58)
        clip = self.core.std.Expr(clip, "x not")
        self.assertEqual(get_pixel_value(clip), 0)

    def test_expr_op53(self):
        clip = self.core.std.BlankClip(format=vs.GRAY8, color=58)
        clip = self.core.std.Expr(clip, "x not not")
        self.assertEqual(get_pixel_value(clip), 1)

    def test_expr_op54(self):
        clip = self.core.std.BlankClip(format=vs.GRAY8, color=58)
        clip = self.core.std.Expr(clip, "x 58 =")
        self.assertEqual(get_pixel_value(clip), 1)
        
    def test_expr_op55(self):
        clip = self.core.std.BlankClip(format=vs.GRAY8, color=3)
        clip = self.core.std.Expr(clip, "x 2 pow")
        self.assertEqual(get_pixel_value(clip), 9)
        
    def test_expr_op56(self):
        clip = self.core.std.BlankClip(format=vs.GRAY8, color=6)
        clip = self.core.std.Expr(clip, "2 x pow")
        self.assertEqual(get_pixel_value(clip), 64)

    def test_expr_op57(self):
        clip1 = self.core.std.BlankClip(format=vs.GRAY8, color=10)
        clip2 = self.core.std.BlankClip(format=vs.GRAY8, color=2)
        clip3 = self.core.std.BlankClip(format=vs.GRAY8, color=3)
        clip = self.core.std.Expr((clip1, clip2, clip3), "x y z swap2 * +")
        self.assertEqual(get_pixel_value(clip), 23)

    def test_expr_op58(self):
        clip1 = self.core.std.BlankClip(format=vs.GRAY8, color=10)
        clip2 = self.core.std.BlankClip(format=vs.GRAY8, color=2)
        clip3 = self.core.std.BlankClip(format=vs.GRAY8, color=3)
        clip = self.core.std.Expr((clip1, clip2, clip3), "10 2 3 swap2 * +")
        self.assertEqual(get_pixel_value(clip), 23)

    def test_expr_op59(self):
        clip1 = self.core.std.BlankClip(format=vs.GRAY8, color=10)
        clip2 = self.core.std.BlankClip(format=vs.GRAY8, color=2)
        clip3 = self.core.std.BlankClip(format=vs.GRAY8, color=3)
        clip = self.core.std.Expr((clip1, clip2, clip3), "x y z swap1 * +")
        self.assertEqual(get_pixel_value(clip), 16)

    def test_expr_op60(self):
        clip1 = self.core.std.BlankClip(format=vs.GRAY8, color=10)
        clip2 = self.core.std.BlankClip(format=vs.GRAY8, color=2)
        clip3 = self.core.std.BlankClip(format=vs.GRAY8, color=3)
        clip = self.core.std.Expr((clip1, clip2, clip3), "10 2 3 swap1 * +")
        self.assertEqual(get_pixel_value(clip), 16)

    def test_expr_op61(self):
        clip1 = self.core.std.BlankClip(format=vs.GRAY8, color=10)
        clip2 = self.core.std.BlankClip(format=vs.GRAY8, color=2)
        clip3 = self.core.std.BlankClip(format=vs.GRAY8, color=3)
        clip = self.core.std.Expr((clip1, clip2, clip3), "x dup0 dup1 dup2 y swap3 z * + + swap / +")
        self.assertEqual(get_pixel_value(clip), 35)

    def test_expr_op62(self):
        clip1 = self.core.std.BlankClip(format=vs.GRAY8, color=10)
        clip2 = self.core.std.BlankClip(format=vs.GRAY8, color=2)
        clip3 = self.core.std.BlankClip(format=vs.GRAY8, color=3)
        clip = self.core.std.Expr((clip1, clip2, clip3), "10 dup0 dup1 dup2 2 swap3 3 * + + swap / +")
        self.assertEqual(get_pixel_value(clip), 35)

    def test_expr_op63(self):
        clip1 = self.core.std.BlankClip(format=vs.GRAY8, color=10)
        clip2 = self.core.std.BlankClip(format=vs.GRAY8, color=2)
        clip3 = self.core.std.BlankClip(format=vs.GRAY8, color=3)
        clip = self.core.std.Expr((clip1, clip2, clip3), "x dup0 10 dup2 y swap3 3 * + + swap / +")
        self.assertEqual(get_pixel_value(clip), 35)

    def helper_sincos(self, op='sin', f=lambda x: math.sin(x)):
        clip = self.core.std.BlankClip(format=vs.GRAYS, color=10, width=1025, height=1024, length=2)
        def init_frame(n, f):
            fout = f.copy()
            arr = fout[0]
            M, N = arr.shape
            for i in range(M):
                for j in range(N):
                    arr[i, j] = (n != 0 or -1) * (i*N + j) * 1e-3
            return fout
        clip = self.core.std.ModifyFrame(clip, clip, init_frame)
        clip2 = self.core.std.Expr(clip, "x %s" % op)
        for n in range(clip2.num_frames):
            f1, f2 = map(lambda c: c.get_frame(n), [clip, clip2])
            arr1, arr2 = f1[0], f2[0]
            for i in range(clip.height):
                for j in range(clip.width):
                    self.assertTrue(abs(arr2[i,j] - f(arr1[i,j])) < 1e-6)
    def test_expr_sin64(self):
        self.helper_sincos('sin', lambda x: math.sin(x))
    def test_expr_cos65(self):
        self.helper_sincos('cos', lambda x: math.cos(x))

        
if __name__ == '__main__':
    unittest.main()



import unittest
import vapoursynth as vs

class LogTestSequence(unittest.TestCase):

    def test_log(self):
        last_message = [None]
        def handler(message_type, message):
            last_message[0] = (message_type, message)

        handle = vs.core.add_log_handler(handler)
        vs.core.log_message(vs.MESSAGE_TYPE_INFORMATION, 'hello!')
        vs.core.remove_log_handler(handle)
        handle = None
        vs.core.log_message(vs.MESSAGE_TYPE_INFORMATION, 'not hello!')

        self.assertEqual(last_message[0], (vs.MESSAGE_TYPE_INFORMATION, 'hello!'))

if __name__ == '__main__':
    unittest.main()



import unittest
import itertools
import vapoursynth as vs
core = vs.core

colorfamilies = (vs.GRAY, vs.YUV, vs.RGB)
intbitdepths = (8, 9, 10, 11, 12, 13, 14, 15, 16)
floatbitdepths = (16, 32)
yuvss = (0, 1, 2)



class ZimgTest(unittest.TestCase):
    def _build_format_ids(self):
        for cfs in colorfamilies:
            for bps in intbitdepths:
                if cfs == vs.YUV:
                    for wss in yuvss:
                        for hss in yuvss:
                            yield core.query_video_format(cfs, vs.INTEGER, bps, wss, hss).id
                else:
                    yield core.query_video_format(cfs, vs.INTEGER, bps, 0, 0).id

        for cfs in colorfamilies:
            for bps in floatbitdepths:
                if cfs == vs.YUV:
                    for wss in yuvss:
                        for hss in yuvss:
                            yield core.query_video_format(cfs, vs.FLOAT, bps, wss, hss).id
                else:
                    yield core.query_video_format(cfs, vs.FLOAT, bps, 0, 0).id

    def test_blank_clip_with_format(self):
        formatids = list(self._build_format_ids())
        for informat, outformat in itertools.product(formatids, formatids):
            try:
                clip = core.std.BlankClip(format=informat)     
                if clip.format.color_family in (vs.YUV, vs.GRAY):
                    clip = core.resize.Bicubic(clip, format=outformat, matrix_in_s="709")
                elif core.get_video_format(outformat).color_family in (vs.YUV, vs.GRAY):
                    clip = core.resize.Bicubic(clip, format=outformat, matrix_s="709")       
                else:
                    clip = core.resize.Bicubic(clip, format=outformat) 
                clip.get_frame(0)
            except vs.Error as e:
                raise RuntimeError(f"Failed to convert from {informat} to {outformat}") from e

if __name__ == '__main__':
    unittest.main()


import gc
import sys
import weakref
import unittest
import contextlib
import multiprocessing
from concurrent.futures import ProcessPoolExecutor

from vapoursynth import core
import vapoursynth as vs


class StubPolicy(vs.EnvironmentPolicy):

    def __init__(self) -> None:
        self._current = None
        self._api = None

    def on_policy_registered(self, special_api):
        self._api = special_api
        self._current = None

    def on_policy_cleared(self):
        assert self._api is not None
        self._current = None

    def get_current_environment(self):
        return self._current

    def set_environment(self, environment):
        self._current = environment


@contextlib.contextmanager
def _with_policy():
    pol = StubPolicy()
    vs.register_policy(pol)
    try:
        yield pol
    finally:
        pol._api.unregister_policy()


test_functions = {}
counter = 0

def _wrap_with(ident):
    func = test_functions[ident]
    return func(EnvironmentTest())


def subprocess_runner(func):
    global counter
    my_counter = counter
    counter += 1
    test_functions[my_counter] = func

    def _wrapper(self):
        with ProcessPoolExecutor(max_workers=1, mp_context=multiprocessing.get_context("spawn")) as executor:
            executor.submit(_wrap_with, my_counter).result()

    return _wrapper


class AnObject:
    pass


class EnvironmentTest(unittest.TestCase):
    @subprocess_runner
    def test_environment_can_retrieve_api(self):
        with _with_policy() as pol:
            _version = vs.__api_version__
            self.assertIsNotNone(pol._api.get_vapoursynth_api((_version.api_major << 16) | _version.api_minor))

    @subprocess_runner
    def test_environment_can_retrieve_core_ptr(self):
        with _with_policy() as pol:
            env = pol._api.create_environment()
            self.assertIsNotNone(pol._api.get_core_ptr(env))

    @subprocess_runner
    def test_environment_use_unsets_environment_on_exit(self):
        with _with_policy() as pol:
            env = pol._api.create_environment()
            wrapped = pol._api.wrap_environment(env)
    
            with self.assertRaises(RuntimeError):
                vs.get_current_environment()
    
            with wrapped.use():
                self.assertEqual(vs.get_current_environment(), wrapped)
    
            with self.assertRaises(RuntimeError):
                vs.get_current_environment()
    
    @subprocess_runner
    def test_environment_use_restores_environment_on_exit(self):
        with _with_policy() as pol:
            env1 = pol._api.create_environment()
            wrapped1 = pol._api.wrap_environment(env1)
    
            env2 = pol._api.create_environment()
            wrapped2 = pol._api.wrap_environment(env2)
    
            with wrapped1.use():
                ce1 = vs.get_current_environment()
    
                with wrapped2.use():
                    self.assertNotEqual(ce1, vs.get_current_environment())
    
                self.assertEqual(ce1, vs.get_current_environment())

    @subprocess_runner
    def test_policy_clearing_runs_callbacks(self):
        f1_run = [False]
        def f1():
            f1_run[0] = True

        f2_run = [False]
        def f2():
            f2_run[0] = True


        with _with_policy() as pol:
            env = pol._api.create_environment()
            wrapped = pol._api.wrap_environment(env)

            with wrapped.use():
                vs.register_on_destroy(f1)
                vs.register_on_destroy(f2)
                vs.unregister_on_destroy(f1)

        self.assertFalse(f1_run[0])
        self.assertTrue(f2_run[0])

        f1_run = [False]
        f2_run = [False]

        with _with_policy() as pol:
            env = pol._api.create_environment()
            wrapped = pol._api.wrap_environment(env)

            with wrapped.use():
                vs.register_on_destroy(f1)

        self.assertTrue(f1_run[0])
        self.assertFalse(f2_run[0])

    @subprocess_runner
    def test_environment_destruction_runs_callbacks(self):
        f1_run = [False]
        def f1():
            f1_run[0] = True

        f2_run = [False]
        def f2():
            f2_run[0] = True


        with _with_policy() as pol:
            env = pol._api.create_environment()
            wrapped = pol._api.wrap_environment(env)

            with wrapped.use():
                vs.register_on_destroy(f1)
                vs.register_on_destroy(f2)
                vs.unregister_on_destroy(f1)

            pol._api.destroy_environment(env)

            self.assertFalse(f1_run[0])
            self.assertTrue(f2_run[0])


            f1_run = [False]
            f2_run = [False]

            env = pol._api.create_environment()
            wrapped = pol._api.wrap_environment(env)

            with wrapped.use():
                vs.register_on_destroy(f1)

            pol._api.destroy_environment(env)

            self.assertTrue(f1_run[0])
            self.assertFalse(f2_run[0])

    @subprocess_runner
    def test_environment_warns_against_resource_leaks(self):
        with _with_policy() as pol:
            env = pol._api.create_environment()

            with self.assertWarnsRegex(RuntimeWarning, "An environment is getting collected"):
                env = None
                gc.collect()


    @subprocess_runner
    def test_locals_store_data_between_envs(self):
        local = vs.Local()

        with _with_policy() as pol:
            env1 = pol._api.create_environment()
            wrapped1 = pol._api.wrap_environment(env1)

            env2 = pol._api.create_environment()
            wrapped2 = pol._api.wrap_environment(env2)

            with wrapped1.use():
                with self.assertRaises(AttributeError):
                    local.hello

            with wrapped2.use():
                local.hello = 5

            with wrapped1.use():
                local.hello = 1

            with wrapped2.use():
                self.assertEqual(local.hello, 5)

            with wrapped1.use():
                self.assertEqual(local.hello, 1)

            with wrapped1.use():
                del local.hello

            with wrapped2.use():
                self.assertEqual(local.hello, 5)

            with wrapped1.use():
                with self.assertRaises(AttributeError):
                    local.hello
    
    @subprocess_runner
    def test_locals_differ_from_each_other(self):
        local1 = vs.Local()
        local2 = vs.Local()

        with _with_policy() as pol:
            env = pol._api.create_environment()
            wrapped = pol._api.wrap_environment(env)

            with wrapped.use():
                local1.a = 5
                local2.a = 6

                self.assertEqual(local1.a, 5)
                self.assertEqual(local2.a, 6)
    
    @subprocess_runner
    def test_locals_store_data_between_envs(self):
        local = vs.Local()
        o = AnObject()

        with _with_policy() as pol:
            env = pol._api.create_environment()
            wrapped = pol._api.wrap_environment(env)

            with wrapped.use():
                local.obj = o

            wr = weakref.ref(o)
            del o

            with wrapped.use():
                self.assertIsNotNone(wr())

            pol._api.destroy_environment(env)
            gc.collect()
            gc.collect()
            gc.collect()
            
            self.assertIsNone(wr())

if __name__ == '__main__':
    unittest.main()


#
# Copyright (c) 2021-2024 Fredrik Mellbin
#
# This file is part of VapourSynth.
#
# VapourSynth is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# VapourSynth is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with VapourSynth; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#

from enum import IntEnum, IntFlag

cdef extern from "include/VSConstants4.h" nogil:
    enum:
        VSC_RANGE_FULL
        VSC_RANGE_LIMITED

    enum:
        VSC_CHROMA_LEFT
        VSC_CHROMA_CENTER
        VSC_CHROMA_TOP_LEFT
        VSC_CHROMA_TOP
        VSC_CHROMA_BOTTOM_LEFT
        VSC_CHROMA_BOTTOM

    enum:
        VSC_FIELD_PROGRESSIVE
        VSC_FIELD_TOP
        VSC_FIELD_BOTTOM

    enum:
        VSC_MATRIX_RGB
        VSC_MATRIX_BT709
        VSC_MATRIX_UNSPECIFIED
        VSC_MATRIX_FCC
        VSC_MATRIX_BT470_BG
        VSC_MATRIX_ST170_M
        VSC_MATRIX_ST240_M
        VSC_MATRIX_YCGCO
        VSC_MATRIX_BT2020_NCL
        VSC_MATRIX_BT2020_CL
        VSC_MATRIX_CHROMATICITY_DERIVED_NCL
        VSC_MATRIX_CHROMATICITY_DERIVED_CL
        VSC_MATRIX_ICTCP

    enum:
        VSC_TRANSFER_BT709
        VSC_TRANSFER_UNSPECIFIED
        VSC_TRANSFER_BT470_M
        VSC_TRANSFER_BT470_BG
        VSC_TRANSFER_BT601
        VSC_TRANSFER_ST240_M
        VSC_TRANSFER_LINEAR
        VSC_TRANSFER_LOG_100
        VSC_TRANSFER_LOG_316
        VSC_TRANSFER_IEC_61966_2_4
        VSC_TRANSFER_IEC_61966_2_1
        VSC_TRANSFER_BT2020_10
        VSC_TRANSFER_BT2020_12
        VSC_TRANSFER_ST2084
        VSC_TRANSFER_ST428
        VSC_TRANSFER_ARIB_B67

    enum:
        VSC_PRIMARIES_BT709
        VSC_PRIMARIES_UNSPECIFIED
        VSC_PRIMARIES_BT470_M
        VSC_PRIMARIES_BT470_BG
        VSC_PRIMARIES_ST170_M
        VSC_PRIMARIES_ST240_M
        VSC_PRIMARIES_FILM
        VSC_PRIMARIES_BT2020
        VSC_PRIMARIES_ST428
        VSC_PRIMARIES_ST431_2
        VSC_PRIMARIES_ST432_1
        VSC_PRIMARIES_EBU3213_E

class ColorRange(IntEnum):
    RANGE_FULL = VSC_RANGE_FULL
    RANGE_LIMITED = VSC_RANGE_LIMITED

class ChromaLocation(IntEnum):
    CHROMA_LEFT = VSC_CHROMA_LEFT
    CHROMA_CENTER = VSC_CHROMA_CENTER
    CHROMA_TOP_LEFT = VSC_CHROMA_TOP_LEFT
    CHROMA_TOP = VSC_CHROMA_TOP
    CHROMA_BOTTOM_LEFT = VSC_CHROMA_BOTTOM_LEFT
    CHROMA_BOTTOM = VSC_CHROMA_BOTTOM
    
class FieldBased(IntEnum):
    FIELD_PROGRESSIVE = VSC_FIELD_PROGRESSIVE
    FIELD_TOP = VSC_FIELD_TOP
    FIELD_BOTTOM = VSC_FIELD_BOTTOM
    
class MatrixCoefficients(IntEnum):
    MATRIX_RGB = VSC_MATRIX_RGB
    MATRIX_BT709 = VSC_MATRIX_BT709
    MATRIX_UNSPECIFIED = VSC_MATRIX_UNSPECIFIED
    MATRIX_FCC = VSC_MATRIX_FCC
    MATRIX_BT470_BG = VSC_MATRIX_BT470_BG
    MATRIX_ST170_M = VSC_MATRIX_ST170_M
    MATRIX_ST240_M = VSC_MATRIX_ST240_M
    MATRIX_YCGCO = VSC_MATRIX_YCGCO
    MATRIX_BT2020_NCL = VSC_MATRIX_BT2020_NCL
    MATRIX_BT2020_CL = VSC_MATRIX_BT2020_CL
    MATRIX_CHROMATICITY_DERIVED_NCL = VSC_MATRIX_CHROMATICITY_DERIVED_NCL
    MATRIX_CHROMATICITY_DERIVED_CL = VSC_MATRIX_CHROMATICITY_DERIVED_CL
    MATRIX_ICTCP = VSC_MATRIX_ICTCP
    
class TransferCharacteristics(IntEnum):
    TRANSFER_BT709 = VSC_TRANSFER_BT709
    TRANSFER_UNSPECIFIED = VSC_TRANSFER_UNSPECIFIED
    TRANSFER_BT470_M = VSC_TRANSFER_BT470_M
    TRANSFER_BT470_BG = VSC_TRANSFER_BT470_BG
    TRANSFER_BT601 = VSC_TRANSFER_BT601
    TRANSFER_ST240_M = VSC_TRANSFER_ST240_M
    TRANSFER_LINEAR = VSC_TRANSFER_LINEAR
    TRANSFER_LOG_100 = VSC_TRANSFER_LOG_100
    TRANSFER_LOG_316 = VSC_TRANSFER_LOG_316
    TRANSFER_IEC_61966_2_4 = VSC_TRANSFER_IEC_61966_2_4
    TRANSFER_IEC_61966_2_1 = VSC_TRANSFER_IEC_61966_2_1
    TRANSFER_BT2020_10 = VSC_TRANSFER_BT2020_10
    TRANSFER_BT2020_12 = VSC_TRANSFER_BT2020_12
    TRANSFER_ST2084 = VSC_TRANSFER_ST2084
    TRANSFER_ST428 = VSC_TRANSFER_ST428
    TRANSFER_ARIB_B67 = VSC_TRANSFER_ARIB_B67

class ColorPrimaries(IntEnum):
    PRIMARIES_BT709 = VSC_PRIMARIES_BT709
    PRIMARIES_UNSPECIFIED = VSC_PRIMARIES_UNSPECIFIED
    PRIMARIES_BT470_M = VSC_PRIMARIES_BT470_M
    PRIMARIES_BT470_BG = VSC_PRIMARIES_BT470_BG
    PRIMARIES_ST170_M = VSC_PRIMARIES_ST170_M
    PRIMARIES_ST240_M = VSC_PRIMARIES_ST240_M
    PRIMARIES_FILM = VSC_PRIMARIES_FILM
    PRIMARIES_BT2020 = VSC_PRIMARIES_BT2020
    PRIMARIES_ST428 = VSC_PRIMARIES_ST428
    PRIMARIES_ST431_2 = VSC_PRIMARIES_ST431_2
    PRIMARIES_ST432_1 = VSC_PRIMARIES_ST432_1
    PRIMARIES_EBU3213_E = VSC_PRIMARIES_EBU3213_E


import unittest
import vapoursynth as vs

class FilterTestSequence(unittest.TestCase):

    def setUp(self):
        self.core = vs.core
        self.Lut = self.core.std.Lut
        self.Lut2 = self.core.std.Lut2
        self.BlankClip = self.core.std.BlankClip
        self.mask = lambda val, bits: val & ((1 << bits) - 1)
		
    def checkDifference(self, cpu, gpu):
        diff = self.core.std.PlaneStats(cpu, gpu, 0, prop="PlaneStats0")
        diff = self.core.std.PlaneStats(diff, gpu, 1, prop="PlaneStats1")
        diff = self.core.std.PlaneStats(diff, gpu, 2, prop="PlaneStats2")

        for i in range(diff.num_frames):
            frame = diff.get_frame(i)
            self.assertEqual(frame.props['PlaneStats0Diff'], 0)
            self.assertEqual(frame.props['PlaneStats1Diff'], 0)
            self.assertEqual(frame.props['PlaneStats2Diff'], 0)

    def testLUT8to16Bit(self):
        clip = self.BlankClip(format=vs.YUV420P8, color=[0, 0, 0])
        ret = self.Lut(clip, planes=[0, 1, 2], lut=[0]*256, bits=16)
        self.checkDifference(ret, self.BlankClip(format=vs.YUV420P16, color=[0, 0, 0]))
            
    def testLUT16Bit(self):
        clip = self.BlankClip(format=vs.YUV420P16, color=[69, 242, 115])
        ret = self.Lut(clip, planes=[0, 1, 2], function=lambda x: x)
        self.checkDifference(clip, ret)

    def testLUT2_8Bit(self):
        clipx = self.BlankClip(format=vs.YUV420P8, color=[69, 242, 115])
        clipy = self.BlankClip(format=vs.YUV420P8, color=[115, 103, 205])

        ret = self.Lut2(clipa=clipx, clipb=clipy, planes=[0, 1, 2], function=lambda x, y: x, bits=8)
        self.checkDifference(clipx, ret)

        ret = self.Lut2(clipa=clipx, clipb=clipy, planes=[0, 1, 2], function=lambda x, y: x, bits=10)
        comp = self.BlankClip(format=vs.YUV420P10, color=[69, 242, 115])
        self.checkDifference(comp, ret)

    def testLUT2_8Bit_10Bit(self):
        # Check 8-bit, 10-bit source.
        clipx = self.BlankClip(format=vs.YUV420P8, color=[69, 242, 115])
        clipy = self.BlankClip(format=vs.YUV420P10, color=[15, 900, 442])

        ret = self.Lut2(clipa=clipx, clipb=clipy, planes=[0, 1, 2], function=lambda x, y: self.mask(x, 8), bits=8)
        self.checkDifference(clipx, ret)

        ret = self.Lut2(clipa=clipx, clipb=clipy, planes=[0, 1, 2], function=lambda x, y: x, bits=10)
        comp = self.BlankClip(format=vs.YUV420P10, color=[69, 242, 115])
        self.checkDifference(comp, ret)

        # Check 10-bit, 8-bit source.
        # Colors are 8-bit levels for 10-bit clip so that we can verify output.
        clipx = self.BlankClip(format=vs.YUV420P10, color=[15, 235, 115])
        clipy = self.BlankClip(format=vs.YUV420P8, color=[69, 242, 115])

        ret = self.Lut2(clipa=clipx, clipb=clipy, planes=[0, 1, 2], function=lambda x, y: self.mask(x, 8), bits=8)
        comp = self.BlankClip(format=vs.YUV420P8, color=[15, 235, 115])
        self.checkDifference(comp, ret)

        ret = self.Lut2(clipa=clipx, clipb=clipy, planes=[0, 1, 2], function=lambda x, y: x, bits=10)
        self.checkDifference(clipx, ret)

if __name__ == '__main__':
    unittest.main()



import os
import subprocess

EXPR_DEBUGGER_PATH = os.path.realpath(os.path.dirname(__file__) + \
                     "../../../msvc_project/x64/Debug/ExprDebugger.exe")

TEST_CASES = ['havs_exprs.txt', 'muvs_exprs.txt']
OUTPUT = 'compiler_output.txt'

def main():
    with open(OUTPUT, 'wb') as o:
        for test_file in TEST_CASES:
            where = os.path.realpath(os.path.dirname(__file__) + "/" +
                                     test_file)
            with open(where, 'r') as file:
                for line in file:
                    result = subprocess.check_output([EXPR_DEBUGGER_PATH, line],
                                                     creationflags=0x08000000)
                    o.write(result)

if __name__ == '__main__':
    main()



import unittest
import vapoursynth as vs

def get_pixel_value(clip, plane):
    frame = clip.get_frame(0)
    arr = frame[plane]
    return arr[0,0]

class FilterTestSequence(unittest.TestCase):

    def setUp(self):
        self.core = vs.core
        self.Transpose = self.core.std.Transpose
        self.BlankClip = self.core.std.BlankClip
        self.MakeFullDiff = self.core.std.MakeFullDiff
        self.MergeFullDiff = self.core.std.MergeFullDiff
		
    def test_transpose8_test(self):
        clip = self.BlankClip(format=vs.YUV420P8, color=[0, 0, 0], width=1156, height=752)
        self.Transpose(clip).get_frame(0)

    def test_transpose16(self):
        clip = self.BlankClip(format=vs.YUV420P16, color=[0, 0, 0], width=1156, height=752)
        self.Transpose(clip).get_frame(0)

    def test_transposeS(self):
        clip = self.BlankClip(format=vs.YUV444PS, color=[0, 0, 0], width=1156, height=752)
        self.Transpose(clip).get_frame(0)

    def test_makefulldiff1(self):
        clipa = self.BlankClip(format=vs.YUV420P8, color=[0, 255, 0], width=1156, height=752)
        clipb = self.BlankClip(format=vs.YUV420P8, color=[255, 0, 0], width=1156, height=752)
        diff1 = self.MakeFullDiff(clipa, clipb)
        newclipb = self.MergeFullDiff(clipb, diff1)
        self.assertEqual(get_pixel_value(newclipb, 0), get_pixel_value(clipa, 0))
        self.assertEqual(get_pixel_value(newclipb, 1), get_pixel_value(clipa, 1))

if __name__ == '__main__':
    unittest.main()
